'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _index = require('../../index');

var _index2 = _interopRequireDefault(_index);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-env mocha */
describe('The database', function () {
  var db = void 0;

  beforeEach(function () {
    db = (0, _index2.default)();
  });

  it('allows document structures', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
    var users, alice, settings, list, user, prefs;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return db.write('users', {});

          case 2:
            users = _context.sent;
            _context.next = 5;
            return db.write('users/alice', {
              name: 'Alice'
            });

          case 5:
            alice = _context.sent;
            _context.next = 8;
            return db.write('settings/alice', {
              background: 'blue'
            });

          case 8:
            settings = _context.sent;
            _context.next = 11;
            return users.write('alice', alice);

          case 11:
            _context.next = 13;
            return alice.write('settings', settings);

          case 13:
            _context.next = 15;
            return db.read('users');

          case 15:
            list = _context.sent;
            _context.next = 18;
            return list.read('alice');

          case 18:
            user = _context.sent;
            _context.next = 21;
            return user.read('settings');

          case 21:
            prefs = _context.sent;


            // Assert the data wrote correctly.
            (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(prefs))).toEqual([['background', 'blue']]);

          case 23:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, undefined);
  })));

  it('allows circular references', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
    var alice, bob, friend;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return db.write('users/alice', {
              name: 'Alice'
            });

          case 2:
            alice = _context2.sent;
            _context2.next = 5;
            return db.write('users/bob', {
              name: 'Bob'
            });

          case 5:
            bob = _context2.sent;
            _context2.next = 8;
            return alice.write('friend', bob);

          case 8:
            _context2.next = 10;
            return bob.write('friend', alice);

          case 10:

            // Assert the links works.
            friend = alice;
            _context2.next = 13;
            return friend.read('friend');

          case 13:
            friend = _context2.sent;
            _context2.next = 16;
            return friend.read('friend');

          case 16:
            friend = _context2.sent;
            _context2.next = 19;
            return friend.read('friend');

          case 19:
            friend = _context2.sent;
            _context2.t0 = _expect2.default;
            _context2.next = 23;
            return friend.read('name');

          case 23:
            _context2.t1 = _context2.sent;
            (0, _context2.t0)(_context2.t1).toBe('Bob');

          case 25:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));

  it('allows concurrent branching', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
    var branch;
    return _regenerator2.default.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return db.write('users/jenn', { firstname: 'Jennifer' });

          case 2:
            branch = db.branch();
            _context3.next = 5;
            return branch.write('users/jenn', { lastname: 'Smith' });

          case 5:
            _context3.next = 7;
            return branch.write('settings/jenn', { notifications: 'ENABLED' });

          case 7:

            // No mutations until commit.
            (0, _expect2.default)(db.value('users/jenn').value('lastname')).toBe(undefined);
            (0, _expect2.default)(db.value('settings/jenn')).toBe(null);

            _context3.next = 11;
            return db.commit(branch);

          case 11:

            (0, _expect2.default)(db.value('users/jenn').value('lastname')).toBe('Smith');
            (0, _expect2.default)(db.value('settings/jenn').value('notifications')).toBe('ENABLED');

          case 13:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, undefined);
  })));
});