'use strict';

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _expect = require('expect');

var _expect2 = _interopRequireDefault(_expect);

var _index = require('../connection-group/index');

var _index2 = _interopRequireDefault(_index);

var _mergeConfigs = require('../merge-configs');

var _mergeConfigs2 = _interopRequireDefault(_mergeConfigs);

var _mocks = require('../mocks');

var _index3 = require('./index');

var pipeline = _interopRequireWildcard(_index3);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var hooks = function hooks(_hooks) {
  return (0, _mergeConfigs2.default)([{ hooks: _hooks }]);
}; /* eslint-env mocha */


describe('The pipeline\'s default option setter', function () {
  var storage = { storage: true };
  var config = void 0,
      group = void 0,
      conn = void 0;

  beforeEach(function () {
    group = new _index2.default();
    conn = new _mocks.Connection();

    group.add(conn);

    config = (0, _mergeConfigs2.default)([{
      storage: [storage],
      network: group
    }]);
  });

  it('adds storage and network fields', function () {
    var options = pipeline.defaults(config, {});

    (0, _expect2.default)(options).toContain({
      storage: [storage],
      network: group
    });
  });

  it('uses given storage options instead of the default', function () {
    var options = pipeline.defaults(config, {
      storage: [{ custom: true }]
    });

    (0, _expect2.default)(options).toContain({
      storage: [{ custom: true }],
      network: group
    });
  });

  it('uses given network options instead of the default', function () {
    var group = new _index2.default();

    var options = pipeline.defaults(config, {
      network: group
    });

    (0, _expect2.default)(options).toContain({
      storage: [storage],
      network: group
    });
  });
});

describe('The before.read.node pipeline', function () {
  it('allows hooks to replace read options', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
    var read, config, options;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            read = function () {
              var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
                return _regenerator2.default.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        return _context.abrupt('return', { overridden: true });

                      case 1:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, undefined);
              }));

              return function read() {
                return _ref2.apply(this, arguments);
              };
            }();

            config = hooks({
              before: {
                read: { node: read }
              }
            });
            _context2.next = 4;
            return pipeline.before.read.node(config, {
              string: 'sup'
            });

          case 4:
            options = _context2.sent;


            (0, _expect2.default)(options).toEqual({ overridden: true });

          case 6:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, undefined);
  })));
});

describe('The pipeline', function () {
  var methods = [['before', 'read', 'node'], ['before', 'read', 'field'], ['before', 'write'], ['before', 'request'], ['before', 'update'], ['after', 'read', 'node'], ['after', 'read', 'field'], ['after', 'write'], ['after', 'request'], ['after', 'update']];

  var buildConfig = function buildConfig(path, fn) {
    var hooksObject = {};

    path.reduce(function (object, field, index) {
      var value = index === path.length - 1 ? fn : {};
      object[field] = value;

      return object[field];
    }, hooksObject);

    return hooks(hooksObject);
  };

  var getPipeline = function getPipeline(path) {
    return path.reduce(function (obj, type) {
      return obj[type];
    }, pipeline);
  };

  methods.forEach(function (path) {
    describe('"' + path.join('.') + '" method', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee8() {
      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              it('allows hooks to override the arguments', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3() {
                var options, hook, config, result;
                return _regenerator2.default.wrap(function _callee3$(_context3) {
                  while (1) {
                    switch (_context3.prev = _context3.next) {
                      case 0:
                        options = { original: 'yep' };

                        hook = function hook(initial) {
                          return (0, _extends3.default)({}, initial, { added: true });
                        };

                        config = buildConfig(path, hook);
                        _context3.next = 5;
                        return getPipeline(path)(config, options);

                      case 5:
                        result = _context3.sent;

                        (0, _expect2.default)(result).toContain((0, _extends3.default)({}, options, { added: true }));

                      case 7:
                      case 'end':
                        return _context3.stop();
                    }
                  }
                }, _callee3, undefined);
              })));

              it('adds default settings', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4() {
                var options, config, result;
                return _regenerator2.default.wrap(function _callee4$(_context4) {
                  while (1) {
                    switch (_context4.prev = _context4.next) {
                      case 0:
                        options = { setting: true };
                        config = hooks();
                        _context4.next = 4;
                        return getPipeline(path)(config, options);

                      case 4:
                        result = _context4.sent;


                        (0, _expect2.default)(result).toEqual({
                          network: new _index2.default(),
                          setting: true,
                          storage: []
                        });

                      case 6:
                      case 'end':
                        return _context4.stop();
                    }
                  }
                }, _callee4, undefined);
              })));

              it('does not force defaults if otherwise specified', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee5() {
                var options, config, result;
                return _regenerator2.default.wrap(function _callee5$(_context5) {
                  while (1) {
                    switch (_context5.prev = _context5.next) {
                      case 0:
                        options = {
                          setting: true,
                          storage: [{ storage: true }],
                          network: [{ network: true }]
                        };
                        config = hooks();
                        _context5.next = 4;
                        return getPipeline(path)(config, options);

                      case 4:
                        result = _context5.sent;


                        (0, _expect2.default)(result).toEqual(options);

                      case 6:
                      case 'end':
                        return _context5.stop();
                    }
                  }
                }, _callee5, undefined);
              })));

              it('uses an empty storage list if null is given', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee6() {
                var options, config, result;
                return _regenerator2.default.wrap(function _callee6$(_context6) {
                  while (1) {
                    switch (_context6.prev = _context6.next) {
                      case 0:
                        options = { storage: null };
                        config = (0, _mergeConfigs2.default)([{
                          storage: { name: 'storage driver' }
                        }]);
                        _context6.next = 4;
                        return getPipeline(path)(config, options);

                      case 4:
                        result = _context6.sent;


                        (0, _expect2.default)(result.storage).toEqual([]);

                      case 6:
                      case 'end':
                        return _context6.stop();
                    }
                  }
                }, _callee6, undefined);
              })));

              it('uses an empty network group if null is given', (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee7() {
                var group, conn, config, result;
                return _regenerator2.default.wrap(function _callee7$(_context7) {
                  while (1) {
                    switch (_context7.prev = _context7.next) {
                      case 0:
                        group = new _index2.default();
                        conn = new _mocks.Connection({ id: 'connection 1' });

                        group.add(conn);

                        config = (0, _mergeConfigs2.default)([{
                          network: group
                        }]);
                        _context7.next = 6;
                        return getPipeline(path)(config, { network: null });

                      case 6:
                        result = _context7.sent;


                        (0, _expect2.default)(result.network).toBeA(_index2.default);
                        (0, _expect2.default)([].concat((0, _toConsumableArray3.default)(result.network))).toEqual([]);

                      case 9:
                      case 'end':
                        return _context7.stop();
                    }
                  }
                }, _callee7, undefined);
              })));

            case 5:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, undefined);
    })));
  });
});